/* Constants.c generated by valac 0.56.19, the Vala compiler
 * generated from Constants.vala, do not modify */

/*
 * Copyright 2019 elementary, Inc. (https://elementary.io)
 *           2015 Adam Bieńkowski
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authored by: Adam Bieńkowski <donadigos159@gmail.com>
 */

#include <stdlib.h>
#include <string.h>
#include <glib.h>

#define PC_CONSTANTS_LOGIN_IFACE "org.freedesktop.login1"
#define PC_CONSTANTS_LOGIN_OBJECT_PATH "/org/freedesktop/login1"
#define PC_CONSTANTS_DBUS_PROPERTIES_IFACE "org.freedesktop.DBus.Properties"
#define PC_CONSTANTS_PARENTAL_CONTROLS_IFACE "org.pantheon.ParentalControls"
#define PC_CONSTANTS_PARENTAL_CONTROLS_OBJECT_PATH "/org/pantheon/ParentalControls"
#define PC_CONSTANTS_PARENTAL_CONTROLS_ACTION_ID "io.elementary.settings.screentime-limits.administration"
#define PC_CONSTANTS_PLANK_CONF_DIR "/.config/plank/dock1/settings"
#define PC_CONSTANTS_DAEMON_CONF_FILE "/etc/pantheon-parental-controls/daemon.conf"
#define PC_CONSTANTS_DAEMON_KEY_ACTIVE "Active"
#define PC_CONSTANTS_DAEMON_KEY_TARGETS "Targets"
#define PC_CONSTANTS_DAEMON_KEY_ADMIN "Admin"
#define PC_CONSTANTS_DAEMON_KEY_BLOCK_URLS "BlockUrls"
#define PC_CONSTANTS_PAM_CONF_START "## PANTHEON_PARENTAL_CONTROLS_START"
#define PC_CONSTANTS_PAM_CONF_END "## PANTHEON_PARENTAL_CONTROLS_END"
#define PC_CONSTANTS_PAM_TIME_CONF_PATH "/etc/security/time.conf"
#define PC_CONSTANTS_CLIENT_PATH "/usr/lib/pantheon-parental-controls-client"
#define PC_CONSTANTS_GETTEXT_PACKAGE "io.elementary.settings.screentime-limits"
#define PC_CONSTANTS_LOCALEDIR "/usr/share/locale"
#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif
#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

const gchar* PC_CONSTANTS_DAEMON_IGNORED_USERS[1] = {"lightdm"};

